package agentrouble;

import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;
import jade.wrapper.*;

import java.util.*;

public abstract class Facilitador extends Agent
{

	public MsgTratador tratador;
	boolean passo;
	
	public final boolean debug = false;
	public abstract Set getOperadores();
	public abstract Set getUsuarios();

	protected void setup()
	{
		tratador = new MsgTratador("facilitador", this);
		
		/*
		 * cria agentes operacao
		 */
		Object[] argumentos = new Object[1];
		argumentos[0] = this.getAID();

		/*
		 * criando operadores
		 */
		for (Iterator i = getOperadores().iterator(); i.hasNext();) {
			Tarefa t = (Tarefa)i.next();

			try {

				jade.wrapper.AgentContainer container = (jade.wrapper.AgentContainer) getContainerController(); 
				AgentController som1 = container.createNewAgent(t.nome(), t.classe(), argumentos);
				som1.start();
			} catch (Exception any) {
				any.printStackTrace();
			}
		}


		/*
		 * criando usuarios
		 */
		for (Iterator i = getUsuarios().iterator(); i.hasNext();) {
			Tarefa t = (Tarefa)i.next();
			try {

				jade.wrapper.AgentContainer container = (jade.wrapper.AgentContainer) getContainerController(); 
				AgentController som1 = container.createNewAgent(t.nome(), t.classe(), argumentos);
				som1.start();
			} catch (Exception any) {
				any.printStackTrace();
			}
		}

		tratador.estouPronto();
		addBehaviour(new ComportamentoFacilitador(this));
	}
	protected void takeDown()
	{
		tratador.terminado();
	}
	
	
	class ComportamentoFacilitador extends Behaviour 
	{
		Inscritor operadores;
		Collection clientes;
		

		public ComportamentoFacilitador(Agent p) {
			super(p);
			operadores = new Inscritor();
			clientes = new TreeSet();
		}

		public void action()
		{

			ACLMessage msgr = tratador.receberObjeto();
			switch(msgr.getPerformative()) {

				/**
				 * inscricao de servico
				 */
				case ACLMessage.SUBSCRIBE: {
					if (msgr.getContent().equals("usuario")) {
						myAgent.send(tratador.geraAceito(msgr));
						clientes.add(msgr.getSender());
					}
					else {
						/*
						 * adicionando no mapa de operadores
						 */
						operadores.inscreve(
								msgr.getContent(), msgr.getSender()
								);
						myAgent.send(tratador.geraAceito(msgr));
					}
					break;
				}

				case ACLMessage.REQUEST: {
					String r = msgr.getContent();
					AID op = (AID) operadores.requer(r);

					if (op == null) {
						if (debug) {
							System.out.println("" + operadores);
							System.out.println("OP NULO");
							System.exit(-3);
						}
						myAgent.send(tratador.geraNegado(msgr));
					}
					else {
						myAgent.send(tratador.geraAceito(msgr));
						myAgent.send(tratador.respondeObjeto(msgr, op));
					}
					break;
				}

				case ACLMessage.INFORM: {
					try {
						AID op = (AID) msgr.getContentObject();
						operadores.libera(op);

						ACLMessage res = tratador.geraAceito(msgr);
						send(res);
					}
					catch (jade.lang.acl.UnreadableException e) {
						ACLMessage res = tratador.geraNegado(msgr);
						send(res);
					}
					break;
				}
			}
		}

		public boolean done() {
			return false;
		}
	}
}
