package agentrouble;

import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

import java.util.*;

public class MsgTratador 
{

	public final boolean debug = false;
	String nome;
	Agent agente;

	public MsgTratador(String n, Agent a) {
		nome = n;
		agente = a;
	}

	protected ACLMessage responde(ACLMessage msgr, String op) {
		ACLMessage fol = msgr.createReply();
		fol.setPerformative(ACLMessage.INFORM);
		fol.setLanguage("ldoi");
		fol.setContent(op);
		mensagem("Enviei resposta",fol.toString());
		return fol;
	}


	protected ACLMessage respondeObjeto(ACLMessage msgr, java.io.Serializable op) {
		try {
			ACLMessage fol = msgr.createReply();
			fol.setPerformative(ACLMessage.INFORM);
			fol.setLanguage("serializable");
			fol.setContentObject(op);
			mensagem("Enviei objeto resposta ",fol.toString());
			return fol;
		}
		catch (java.io.IOException e) {
			e.printStackTrace();
			System.exit(-1);
			return null;
		}
	}

	protected ACLMessage geraAceito(ACLMessage msgr) {
		ACLMessage rep = msgr.createReply();
		rep.setPerformative(ACLMessage.INFORM);
		rep.setContent("aceito");
		mensagem("Respondi requisicao",rep.toString());
		return rep;
	}
	protected ACLMessage geraNegado(ACLMessage msgr) {
		ACLMessage rep = msgr.createReply();
		rep.setPerformative(ACLMessage.INFORM);
		rep.setContent("negado");
		mensagem("Respondi requisicao",rep.toString());
		return rep;
	}

	protected void estouPronto() {
		mensagem("Estou pronto");
	}

	protected void terminado() {
		mensagem("Terminei");
	}

	protected ACLMessage receberObjeto() {
		mensagem("VOU ESPERAR RESPOSTA: "+agente.getAID());
		ACLMessage msgr = agente.blockingReceive();
		mensagem("Recebi objeto");
		return msgr;
	}

	protected ACLMessage receber() {
		mensagem("VOU ESPERAR RESPOSTA: "+agente.getAID());
		ACLMessage msgr = agente.blockingReceive();
		mensagem("Recebi", msgr.getContent());
		return msgr;
	}

	private void mensagem(String s) {
		if (debug)
			System.out.println("------------- "+agente.getAID().getName()+": "+s+".");
	}
	
	private void mensagem(String s, String s2) {
		if (debug)
			System.out.println("------------- "+agente.getAID().getName()+": "+s+": {\n" +s2+"}");
	}

	protected ACLMessage inscricao(AID facilitador, String tipo) {
		ACLMessage msg = new ACLMessage(ACLMessage.SUBSCRIBE);

		msg.setContent(tipo);
		msg.addReceiver(facilitador);
		msg.setLanguage("lii"); /* linguagem de inscricao improvisada */
		msg.setOntology("inscricao");

		agente.doWait(1000);

		mensagem("Vou me inscrever", msg.toString());

		return msg;
	}

	protected ACLMessage informarObjeto(AID facilitador, java.io.Serializable operador) {
		try {
			ACLMessage msg = new ACLMessage(ACLMessage.INFORM);

			msg.setContentObject(operador);
			msg.addReceiver(facilitador);
			msg.setLanguage("selializable"); /* linguagem de inscricao improvisada */
			msg.setOntology("operacao");

			mensagem("Informei objeto", msg.toString());

			return msg;
		}
		catch (java.io.IOException e) {
			e.printStackTrace();
			System.exit(-1);
			return null;
		}
	}


	protected ACLMessage requisitar(AID facilitador, String operador) {
		ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);

		msg.setContent(operador);
		msg.addReceiver(facilitador);
		msg.setLanguage("lri"); /* linguagem de inscricao improvisada */
		msg.setOntology("operacao");

		mensagem("Requisitei", msg.toString());

		return msg;
	}

	protected ACLMessage requisitarObjeto(AID facilitador, java.io.Serializable operacao) {
		try {
			ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);

			msg.setContentObject(operacao);
			msg.addReceiver(facilitador);
			msg.setLanguage("serializable"); /* linguagem de inscricao improvisada */
			msg.setOntology("operacao");

			mensagem("Requisitei objeto", msg.toString());

			return msg;
		}
		catch (java.io.IOException e) {
			e.printStackTrace();
			System.exit(-1);
			return null;
		}
	}
}
