package agentrouble;

import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.*;

import java.util.*;

public abstract class Usuario extends Agent
{
	AID facilitador;
	MsgTratador tratador;
	
	public static final boolean debug = true;

	public abstract Operacao esperaAcao();
	public abstract void recebeResposta(Object o);
	public abstract boolean terminou();
	public abstract void falhou();
	public abstract void inicializa();
	
	protected void setup()
	{
		inicializa();
		tratador =new MsgTratador("usuario", this);
		
		Object[] o = this.getArguments();
		facilitador = (AID) o[0];
		
		ACLMessage msg = tratador.inscricao(facilitador, "usuario");
		this.send(msg);

		ACLMessage rep = tratador.receber();
		if (rep.getContent().equals("aceito")) {

			tratador.estouPronto();
			addBehaviour(new ComportamentoUsuario(this));
		}

	}
	protected void takeDown()
	{
		tratador.terminado();
	}
	
	
	class ComportamentoUsuario extends Behaviour 
	{
		Usuario usuario;
		
		public ComportamentoUsuario(Usuario p) {
			super(p);
			usuario = p;
		}

		public void action()
		{
			Operacao o = usuario.esperaAcao();

			/* envia requisicao */
			ACLMessage aclm = tratador.requisitar(facilitador, o.operador());
			
			myAgent.send(aclm);
			

			/* espera resposta */
			ACLMessage rsp = tratador.receber();

			if (rsp.getContent().equals("aceito")) {
				// aceito 
				ACLMessage fol = tratador.receberObjeto();
				/* requisitando operacao */
				try {

					/*
					 * inicio da requisicao ao operador
					 */
					AID op = (AID) fol.getContentObject();

					ACLMessage rop = tratador.requisitarObjeto(op, o);
					myAgent.send(rop);

					ACLMessage res = tratador.receber();

					if (res.getContent().equals("negado")) {
						usuario.falhou();
					}
					else {
						ACLMessage rrr = tratador.receberObjeto();
						usuario.recebeResposta(rrr.getContentObject());
					}
					/*
					 * fim da requisicao do operador
					 */

					/*
					 * liberando operador
					 */

					ACLMessage lib = tratador.informarObjeto(facilitador, op);
					myAgent.send(lib);

					ACLMessage conf = tratador.receber();

					if (conf.getContent().equals("negado")) {
						System.out.println("no consegui liberar");
						System.exit(-2);
					}
					

				}
				catch (jade.lang.acl.UnreadableException e) {
					e.printStackTrace();
					usuario.falhou();
				}
			}
			else {
				usuario.falhou();
			}
			
		}

		public boolean done()
		{
			return usuario.terminou();
		}
	}
}
